
from random import random
from sbx.tools.pens import *
import AppKit
import vanilla
from mojo.roboFont import RFont, RGlyph, RLayer
from getDesignSpaceWindow import getSkateboard, getSkateboardDesignSpace
from mojo.events import addObserver, publishEvent, removeObserver
from mojo.UI import MultiLineView
from mojo.canvas import CanvasGroup
import mojo.drawingTools as ctx


class SequencerWindow(object):


    _sequencer_observes_these = [
        ("docChangedCallback", 'skateboard.previewLocation.changed'),
        ("docChangedCallback", "skateboard.interactionSources.changed"),
        ("docChangedCallback", "skateboard.window.opened"),
        ("docChangedCallback", "skateboard.window.closed"),
        ("docChangedCallback", "skateboard.document.opened"),
        ("docChangedCallback", "skateboard.document.closed"),
        ]

    def __init__(self):
        self.textRedrawTimer = None
        self._captionTextFont = "Menlo-Regular"
        self._margin = 10
        self._direction = 1    # 1 = LTR, -1 = RTL
        self._lineSpace = 1.10    # factor of em ?
        self.drawingOK = False
        self.previewFonts = []
        self.glyphNames = ["A", "B", "C"]
        self.colors = {
            'Instance': (0.1, 0.8, 1),
            'Source': (0.8, 0.1, 1),
            'Support': (0.8, 0.8, 1),
            'Open': (0.8, 0.2, 0.5),
            'Interesting': (0.8, 0.8, 0.8),
            }
        self.sb = None
        self.ds = None
        self.defaultFont = None
        self.w = vanilla.Window((500, 300), 
            "Skateboard Sequencer", 
            minSize=(500,300),
            autosaveName = "com.letterror.skateboard.sequencerWindow",
            )

        self.w.c = CanvasGroup((0,30,0,0), delegate=self)
        self.w.direction = vanilla.CheckBox((-50, 5, 50, 20), "LTR", callback=self.directionChangeCallback)
        self.w.refreshButton = vanilla.Button((10, 5, 100, 20), "Refresh", self.refreshCallback)
        self.w.bind("close", self.cbClose)
        self.w.bind("became main", self.docChangedCallback)
        self.w.open()
        #publishEvent("skateboard.running")
        for methodName, notificationName in self._sequencer_observes_these:
            addObserver(self, methodName, notificationName)
        self.checkForSkateboard()
    
    def directionChangeCallback(self, sender):
        print("change direction check", sender.get())
        if sender.get()==1:
            self._direction = -1
        else:
            self._direction = 1
        self.timerSchedule_textWindowRedraw()
        
    def cbClose(self, sender=None):
        for methodName, notificationName in self._sequencer_observes_these:
            removeObserver(self, notificationName)
        
    def docChangedCallback(self, notification=None):
        print("docChangedCallback")
        self.refreshFonts()
        self.timerSchedule_textWindowRedraw()

    def draw(self):
        if self.sb is None:
            print("no skateboard found")
            return

        w = self.w.c.width() - 2 * self._margin
        h = self.w.c.height() - 2 * self._margin
        
        # how many instances
        # really should show the selected items in the sb list.
        lines = len(self.sb.allSelectedLocations)
        if lines == 0:
            print('no selected items')
            return
        #print("lines", lines)
        #print('---------- self.defaultFont', self.defaultFont)
        designspaceEm = self.defaultFont.info.unitsPerEm
        if designspaceEm is None:
            designspaceEm = 1000
        designspaceDescender = self.defaultFont.info.descender
        designspaceAscender = self.defaultFont.info.ascender
        if designspaceDescender is None:
            designspaceDescender = -200
        if designspaceAscender is None:
            designspaceAscender = 800
        lineHeight = h / lines
        detailsOk = lineHeight > 100
        y = h-lineHeight
        if self._direction == -1:
            x = w-2*self._margin
        else:
            x = self._margin

        # draw things
        ctx.save()    #1
        ctx.translate(self._margin, self._margin)
        lineScale = (lineHeight / self._lineSpace) / designspaceEm
        fonts = []
        for a, item in zip(self.previewFonts, self.sb.allSelectedLocations):
            loc, thisFont = a
            flavor = item['flavor']
            location = item['location']
            #print("draw", id(thisFont), thisFont.keys())

            #if flavor in self.colors:
            #    ctx.fill(*self.colors[flavor])
            #    ctx.rect(0,y,w, lineHeight)

            #ctx.fill(1)
            tw, th = ctx.textSize(item.get('description'))
            if self._direction == -1:
                pos = w-self._margin - tw, y
            else:
                pos = 0, y
            #dummyWidth = 500
            if detailsOk:
                ctx.save() #2a
                ctx.fill(0.7)
                ctx.font(self._captionTextFont, 10)
                ctx.text(item.get('description'), pos)
                ctx.restore() #2a
            
            ctx.save() #2
            ctx.fill(None)
            ctx.translate(0,y)
            if self._direction == -1:
                #print("----->")
                ctx.translate(w,0)
            #else:
            #    print("<-----")
            ctx.scale(lineScale)
            # we're on the baseline
            ctx.translate(0,-1.5*designspaceDescender)
            c = 0
            for glyphName in self.glyphNames:
                if not glyphName in thisFont:
                    #print("xxx", glyphName, thisFont.keys())
                    # draw missing glyph
                    continue
                p = PreviewPen(thisFont)
                thisGlyph = thisFont[glyphName]
                d = thisGlyph.width
                ctx.save() #10
                if self._direction == -1:
                    ctx.translate(-d,0)
                #ctx.fill(1,0,0)
                ctx.fill(None)
                ctx.stroke(None)
                ctx.rect(0,0,1,1)
                ctx.fill(0)
                thisGlyph.draw(p)
                ctx.drawPath()
                #ctx.fill(None)
                #ctx.stroke(None)
                ctx.restore() #10
                if self._direction == -1:
                    ctx.translate(-d,0)
                else:
                    ctx.translate(d,0)
                c += 1
            ctx.restore() #2
            y -= lineHeight
        ctx.restore()    #1
    
    def refreshCallback(self, sender=None):
        # vanilla callback for recalculating the previews.
        print('refreshCallback')
        self.refreshFonts()
        if self.sb is not None:
            for loc, previewFont in self.previewFonts:
                self.sb.data_calculateGlyphsForLine(previewFont, loc, self.glyphNames)
                self.glyphNames = self.sb.textGlyphNames
        self.w.c.update()
        
    def refreshFonts(self):
        if self.sb is None:
            self.previewFonts = []
            return
        for loc, f in self.previewFonts:
            f.close()
        self.previewFonts = []
        for item in self.sb.allSelectedLocations:
            previewFont = RFont(showInterface=False)
            self.previewFonts.append((item['location'], previewFont))
        print("refreshFonts self.previewFonts loaded", len(self.previewFonts))

    def checkForSkateboard(self):
        self.sb = getSkateboard()
        self.ds = getSkateboardDesignSpace()
        self.defaultFont = None
        if self.ds is not None:
            self.allFonts = self.ds.getFonts()
            defaultLoc = self.ds.newDefaultLocation(bend=True)
            for font, loc in self.allFonts:
                if defaultLoc == loc:
                    # this should be the default font
                    self.defaultFont = font

        else:
            self.defaultFont = None
        print("checkForSkateboard", self.sb, self.ds)
        self.timerSchedule_textWindowRedraw()

    def timerSchedule_textWindowRedraw(self, debugTag=None):
        # timer for redraws of the text window
        # should be hesitant
        # calls timerDone_textWindow_redrawCallback_
        if self.textRedrawTimer is not None:
            self.textRedrawTimer.invalidate()
            self.textRedrawTimer = None
        interval = .5
        self.textRedrawTimer = AppKit.NSTimer.scheduledTimerWithTimeInterval_target_selector_userInfo_repeats_(
            interval,
            self,
            "timerDone_textWindowRedraw:",
            debugTag, False)
    
    def timerDone_textWindowRedraw_(self, timer):
        # called by timerSchedule_textWindowRedraw
        debugTag = timer.userInfo()
        if debugTag is not None:
            print("timerDone_textWindowRedraw_", debugTag)
        if self.textRedrawTimer is not None:
            self.textRedrawTimer.invalidate()
            self.textRedrawTimer = None
        # redraw text window
        self.refreshCallback()

OpenWindow(SequencerWindow)