
# from fonttools.varlib
# but with optional canExtrapolate flag

def normalizeValue(v, triple, canExtrapolate=False):
    """Normalizes value based on a min/default/max triple.
    >>> normalizeValue(400, (100, 400, 900))
    0.0
    >>> normalizeValue(100, (100, 400, 900))
    -1.0
    >>> normalizeValue(650, (100, 400, 900))
    0.5

    # lower == default < upper
    >>> normalizeValue(-1000, (0, 0, 1000), canExtrapolate=True)
    -1.0
    >>> normalizeValue(-1000, (0, 0, 1000), canExtrapolate=False)
    0.0
    >>> normalizeValue(0, (0, 0, 1000), canExtrapolate=True)
    0.0
    >>> normalizeValue(0, (0, 0, 1000), canExtrapolate=False)
    0.0
    >>> normalizeValue(1000, (0, 0, 1000), canExtrapolate=True)
    1.0
    >>> normalizeValue(1000, (0, 0, 1000), canExtrapolate=False)
    1.0
    >>> normalizeValue(2000, (0, 0, 1000), canExtrapolate=True)
    2.0
    >>> normalizeValue(2000, (0, 0, 1000), canExtrapolate=False)
    1.0

    # lower < default < upper
    >>> normalizeValue(500, (0, 500, 1000), canExtrapolate=True)
    0.0
    >>> normalizeValue(500, (0, 500, 1000), canExtrapolate=False)
    0.0
    >>> normalizeValue(250, (0, 500, 1000), canExtrapolate=True)
    -0.5
    >>> normalizeValue(250, (0, 500, 1000), canExtrapolate=False)
    -0.5
    >>> normalizeValue(0, (0, 500, 1000), canExtrapolate=True)
    -1.0
    >>> normalizeValue(0, (0, 500, 1000), canExtrapolate=False)
    -1.0
    >>> normalizeValue(-500, (0, 500, 1000), canExtrapolate=True)
    -2.0
    >>> normalizeValue(-500, (0, 500, 1000), canExtrapolate=False)
    -1.0
    """
    lower, default, upper = triple
    assert lower <= default <= upper, "invalid axis values: %3.3f, %3.3f %3.3f"%(lower, default, upper)
    if not canExtrapolate:
        v = max(min(v, upper), lower)
    if v < lower:
        if default == lower:
            #     v     ld------------u
            v = (v - lower) / (upper - lower)
        else:
            #     v     l------d------u
            v = (v - default) / (default - lower)
    elif v == default:
        v = 0.
    elif v < default:
        v = (v - default) / (default - lower)
    else:
        if upper == default:
            #      l------------du    v
            v = (v - upper ) / (upper - lower)
        else:
            #      l-------d-----u     v
            v = (v - default) / (upper - default)
    return v

def normalizeLocation(location, axes, canExtrapolate = False):
    """Normalizes location based on axis min/default/max values from axes.
    >>> axes = {"wght": (100, 400, 900)}
    >>> normalizeLocation({"wght": 400}, axes)
    {'wght': 0.0}
    >>> normalizeLocation({"wght": 100}, axes)
    {'wght': -1.0}
    >>> normalizeLocation({"wght": 900}, axes)
    {'wght': 1.0}
    >>> normalizeLocation({"wght": 650}, axes)
    {'wght': 0.5}
    >>> normalizeLocation({"wght": 1000}, axes)
    {'wght': 1.0}
    >>> normalizeLocation({"wght": 1000}, axes, canExtrapolate=True)
    {'wght': 1.2}
    >>> normalizeLocation({"wght": 0}, axes)
    {'wght': -1.0}
    >>> normalizeLocation({"wght": 0}, axes, canExtrapolate=True)
    {'wght': -1.3333333333333333}
    
    >>> axes = {"wght": (0, 0, 1000)}
    >>> normalizeLocation({"wght": 0}, axes)
    {'wght': 0.0}
    >>> normalizeLocation({"wght": -1}, axes)
    {'wght': 0.0}
    >>> normalizeLocation({"wght": -1}, axes, canExtrapolate=True)
    {'wght': -0.001}
    >>> normalizeLocation({"wght": 1000}, axes)
    {'wght': 1.0}
    >>> normalizeLocation({"wght": 500}, axes)
    {'wght': 0.5}
    >>> normalizeLocation({"wght": 1001}, axes)
    {'wght': 1.0}
    >>> normalizeLocation({"wght": 1001}, axes, canExtrapolate=True)
    {'wght': 1.001}
    
    >>> axes = {"wght": (0, 1000, 1000)}
    >>> normalizeLocation({"wght": 0}, axes)
    {'wght': -1.0}
    >>> normalizeLocation({"wght": -1000}, axes)
    {'wght': -1.0}
    >>> normalizeLocation({"wght": -1000}, axes, canExtrapolate=True)
    {'wght': -2.0}
    >>> normalizeLocation({"wght": -1}, axes)
    {'wght': -1.0}
    >>> normalizeLocation({"wght": -1}, axes, canExtrapolate=True)
    {'wght': -1.001}
    >>> normalizeLocation({"wght": 500}, axes)
    {'wght': -0.5}
    >>> normalizeLocation({"wght": 1000}, axes)
    {'wght': 0.0}
    >>> normalizeLocation({"wght": 1001}, axes)
    {'wght': 0.0}
    >>> normalizeLocation({"wght": 1001}, axes, canExtrapolate=True)
    {'wght': 0.001}
    >>> normalizeLocation({"wght": 2000}, axes, canExtrapolate=True)
    {'wght': 1.0}
    """
    out = {}
    for tag,triple in axes.items():
        v = location.get(tag, triple[1])
        out[tag] = normalizeValue(v, triple, canExtrapolate=canExtrapolate)
    return out
    
if __name__ == "__main__":
    import doctest, sys
    doctest.testmod()
