import mojo.drawingTools as ctx
from fontTools.pens.basePen import BasePen
from fontTools.pens.transformPen import TransformPen
import sbx.tools

class CollectorPen(BasePen):
    def __init__(self, glyphSet, path=None):
        self.offset = 0,0
        self.onCurves = []
        self.offCurves = []
        self.startPoints = []
        self._pointIndex = 0
        self.tag = "untagged"
        BasePen.__init__(self, glyphSet)
    def setOffset(self, x=0,y=0):
        self.offset = x, y
    def _moveTo(self, pos):
        self.onCurves.append((pos[0]+self.offset[0], pos[1]+self.offset[1]))
        self.startPoints.append(self._pointIndex)
        self._pointIndex += 1
    def _lineTo(self, pos):
        self.onCurves.append((pos[0]+self.offset[0], pos[1]+self.offset[1]))
        self._pointIndex += 1
    def _curveToOne(self, a, b, c):
        self.offCurves.append((a[0]+self.offset[0], a[1]+self.offset[1]))
        self.offCurves.append((b[0]+self.offset[0], b[1]+self.offset[1]))
        self.onCurves.append((c[0]+self.offset[0], c[1]+self.offset[1]))
        self._pointIndex += 3
    def _closePath(self):
        pass

class PreviewPen(BasePen):
    def __init__(self, glyphSet=None, path=None):
        if glyphSet is None:
            glyphSet = {}
        self.glyphSet = glyphSet
        BasePen.__init__(self, glyphSet)
    def _moveTo(self, p1):
        ctx.moveTo(p1)
    def _lineTo(self, p1):
        ctx.lineTo(p1)
    def _curveToOne(self, p1, p2, p3):
        ctx.curveTo(p1, p2, p3)
    def _closePath(self):
        ctx.closePath()
    def addComponent(self, glyphName, transformation):
        # draw the component here
        if glyphName not in self.glyphSet: return
        g = self.glyphSet[glyphName]
        t = TransformPen(self, transformation)
        g.draw(t)

class ContourMarkingPreviewPen(BasePen):
    # give each contour a new color
    def __init__(self, glyphSet=None, alpha=0.05, markerSize=10, reversedScale=1, showMarkers=False):
        self.drawingComponent = 0
        self.hasPath = False
        self.alpha = alpha
        self.reversedScale = reversedScale
        self.markerSize = 5
        self.showMarkers = showMarkers
        if glyphSet is None:
            glyphSet = {}
        self.glyphSet = glyphSet
        self.paths = []
        self.currentPath = []
        BasePen.__init__(self, glyphSet)
    
    def _flush(self):
        if self.currentPath:
            self.paths.append(self.currentPath)
        self.currentPath = []

    def _moveTo(self, p1):
        self._flush()
        self.currentPath.append(('m', p1))

    def _lineTo(self, p1):
        self.currentPath.append(('l', p1))

    def _curveToOne(self, p1, p2, p3):
        self.currentPath.append(('c', p1, p2, p3))

    def _closePath(self):
        self._flush()
        self.currentPath.append(('x'))

    def addComponent(self, glyphName, transformation):
        # draw the component here
        if glyphName not in self.glyphSet: return
        g = self.glyphSet[glyphName]
        self.drawingComponent += 1
        t = TransformPen(self, transformation)
        g.draw(t)
        self.drawingComponent -= 1
    
    def marker(self, p, c, makeBig=False):
        if not self.showMarkers: return
        ff = 0.8
        if makeBig:
            # starting point?
            ms = self.markerSize * self.reversedScale * 2
            ctx.stroke(None)
            ctx.fill(c[0]*ff, c[1]*ff, c[2]*ff, 1)    #self.alpha)
        else:
            ms = self.markerSize * self.reversedScale
            ctx.fill(c[0]*ff, c[1]*ff, c[2]*ff, 1)
            ctx.stroke(None)
        ctx.oval(p[0]-.5*ms, p[1]-.5*ms, ms, ms)
        
    def paint(self):
        self._flush()
        steps = len(self.paths)
        for i, path in enumerate(self.paths):
            c = sbx.tools.getColor(steps*3, 2+i, saturation=1)
            #fill(None)
            ff = 0.8
            for cmd in path:
                if cmd[0]=='m':
                    ctx.stroke(c[0], c[1], c[2], 0.6)    #self.alpha)
                    if self.drawingComponent == 0:
                        ctx.fill(c[0]*ff, c[1]*ff, c[2]*ff, 0.3)    #self.alpha)
                    else:
                        ctx.fill(None)
                    ctx.newPath()
                    ctx.moveTo(cmd[1])
                elif cmd[0]=='l':
                    ctx.lineTo(cmd[1])
                elif cmd[0]=='c':
                    ctx.curveTo(cmd[1], cmd[2], cmd[3])
                elif cmd[0]=='x':
                    ctx.closePath()
                    ctx.drawPath()
        for i, path in enumerate(self.paths):
            c = sbx.tools.getColor(steps*2, 2+i, saturation=1)
            ctx.stroke(None)
            for cmd in path:
                if cmd[0]=='m':
                    self.marker(cmd[1], c, True)
                elif cmd[0]=='l':
                    self.marker(cmd[1], c)
                elif cmd[0]=='c':
                    #self.marker(cmd[1], c)
                    #self.marker(cmd[2], c)
                    self.marker(cmd[3], c)
        
