# -*- coding: UTF-8 -*-
from __future__ import print_function, absolute_import

import glyphNameFormatter.reader


def splitText(text, fallback=".notdef"):
    """
    Break a string of characters or / delimited glyph names
    into a list.

    - Test name compiling
    >>> splitText("/a")
    ['a']
    >>> splitText("/aacute/bbreve")
    ['aacute', 'bbreve']
    >>> splitText("/aacute /bbreve")
    ['aacute', 'bbreve']
    >>> splitText("/cmb:dieresis /cmb:acute")
    ['cmb:dieresis', 'cmb:acute']
    >>> splitText("/cmb-dieresis /cmb-acute")
    ['cmb-dieresis', 'cmb-acute']

    - Test character input
    >>> splitText("*.")
    ['asterisk', 'period']
    >>> splitText("*.", {42:"asterisk", 46:"period"})
    ['asterisk', 'period']

    - Test slash escaping
    >>> splitText("//")
    ['slash']
    >>> splitText("///")
    ['slash']
    >>> splitText("////")
    ['slash', 'slash']
    >>> splitText("/ /")
    []
    >>> splitText("/ /")
    []
    >>> splitText("1//2", {49:"one", 50:"two"})
    ['one', 'slash', 'two']

    - Test mixture
    >>> splitText("*/aacute .%//", {42:"asterisk", 46:"period"})
    ['asterisk', 'aacute', 'period', 'percent', 'slash']
    """
    # escape //
    text = text.replace("//", "/slash ")
    #
    glyphNames = []
    compileStack = None
    for c in text:
        # start a glyph name compile.
        if c == "/":
            # finishing a previous compile.
            if compileStack is not None:
                # only add the compile if something has been added to the stack.
                if compileStack:
                    glyphNames.append("".join(compileStack))
            # reset the stack.
            compileStack = []
        # adding to or ending a glyph name compile.
        elif compileStack is not None:
            # space. conclude the glyph name compile.
            if c == " ":
                # only add the compile if something has been added to the stack.
                if compileStack:
                    glyphNames.append("".join(compileStack))
                compileStack = None
            # add the character to the stack.
            else:
                compileStack.append(c)
        # adding a character that needs to be converted to a glyph name.
        else:
            #v = ord(c)
            #print("\t", c, v)
            glyphName = glyphNameFormatter.reader.uni2name.get(ord(c))
            if glyphName is None:
                # format at uniXXXX
                glyphName = 'uni%X' %ord(c)
                #glyphName = fallback
            glyphNames.append(glyphName)
    # catch remaining compile.
    if compileStack is not None and compileStack:
        glyphNames.append("".join(compileStack))
    return glyphNames


if __name__ == "__main__":
    assert splitText('%') == ['percent']
    assert splitText('\n') == ['uniA']    