import itertools
from mutatorMath.objects.mutator import Location, buildMutator
    
"""

     + sort the (master) locations to a sequence.
     + prioritise certain classes of locations
         1. origin
         2. on-axis, outward to in
         3. aligned off-axis
         4. aligned on-axis
     + predict where we need to freeze the designspace
       before insering a new location.
     
    Assumptions:
    +    the system does not have any special knowledge
         about specific constructions. So it can't have
         specific solutions for specific problems.
    +    it must be able to find order in any combination of locations
    +    neutral is normalised to 0 on any axis.
    +    normalisation to -1, 0, 1 not required
        
        
"""

def getAxisValues(items, ignoreZero=False):
    positions = {}
    locTuples = []
    for loc in items:
        for axName, axValue in loc.items():
            if not axName in positions:
                positions[axName] = []
            if not axValue in positions[axName]:
                if ignoreZero and axValue == 0: continue
                positions[axName].append(axValue)
                positions[axName].sort()
    return positions

#t1 = [Location(a=1), Location(a=0), Location(b=1), Location(b=2), Location(b=0)]
#assert getAxisValues(t1) == {'a': [0, 1], 'b': [0, 1, 2]}
#assert getAxisValues(t1, ignoreZero=True) == {'a': [1], 'b': [1, 2]}

def findOpenNodes(items, axisOrder):
    # 1. collect axisvalues for these locations
    # 2. list all possible combinations
    # 3. remove existing locations from list
    positions = getAxisValues(items)
    relevantAxisOrder = [axisName for axisName in axisOrder if axisName in positions]
    combs = set(itertools.product(*[positions[k] for k in relevantAxisOrder]))
    for loc in items:
        t = tuple([loc[k] for k in relevantAxisOrder])
        if t in combs:
            combs.remove(t)
        else:
            print("XXX findOpenNodes", t, combs)
    return [Location(dict(zip(axisOrder, item))) for item in combs]
    
def testOnAxis(loc):
    # determines if loc is on axis
    # and then also reports which axis
    # really should check with the default value of axes
    # but let's assume 0 for now
    zero = []
    nonZero = []
    for k, v in loc.items():
        if v != 0:
            nonZero.append(k)
        else:
            zero.append(k)
    if len(nonZero)==1:
        return nonZero[0]
    return None

def sortLocations(items, axisOrder=None, verbose=False):
    # sort the locations as follows:
    #    neutral
    #    on-axis
    #    aligned off-axis
    #    partially aligned on-axis
    #    partially aligned off-axis
    #    non-aligned off-axis, sorted by distance from neutral
    #
    # 
    # also looking at the axis order
    origin = None
    sortedOnAxis = {}
    offAxis = []
    allOnAxis = []
    allOffAxis = []
    aligned_offAxis = []
    partiallyAligned_onAxis = []    # XXX how to find?
    partiallyAligned_offAxis = []
    nonAligned_offAxis = []
    ranking = []
    annotatedRanking = []
    for loc in items:
        if loc.isOrigin():
            origin = loc
            if not axisOrder:
                axisOrder = loc.keys()
                axisOrder.sort()
            continue
        this = testOnAxis(loc)
        if this is not None:
            allOnAxis.append(loc)
            if not this in sortedOnAxis:
                sortedOnAxis[this] = []
            sortedOnAxis[this].append(loc)
            sortedOnAxis[this].sort()
            continue
        offAxis.append(loc)
    if offAxis:
        onAxisPositions = getAxisValues(allOnAxis)
        for loc in offAxis:
            ok = []
            for k, v in loc.items():
                if k in onAxisPositions:
                    if v not in onAxisPositions[k]:
                        ok.append(False)
                    else:
                        ok.append(True)
            if ok and not False in ok:
                aligned_offAxis.append(loc)
            else:
                if True in ok:
                    partiallyAligned_offAxis.append(loc)
                else:
                    nonAligned_offAxis.append(loc)

    # now we have an overview of all the locations in this system.
    # we can prioritise and sort.
    # can we find partially aligned on-axis here?
    offAxisValues = getAxisValues(offAxis, ignoreZero=False) #XXX not sure about this
    if verbose:
        print("offAxisValues", offAxisValues)
    remove = []
    for axisName in axisOrder:
        for loc in sortedOnAxis.get(axisName, []):
            ok = []
            for axisName, value in loc.items():
                if not axisName in offAxisValues:
                    continue
                if not value in offAxisValues[axisName]:
                    ok.append(False)
                else:
                    ok.append(True)
            if not True in ok:
                partiallyAligned_onAxis.append(loc)
                remove.append(loc)
    for loc in remove:
        for axisName in axisOrder:
            if loc in sortedOnAxis.get(axisName,[]):
                sortedOnAxis[axisName].remove(loc)
        
    # make a list in which all the location are in the right order
    # step 1: origin
    addedSupports = []
    if origin is not None:
        ranking.append(origin)
        annotatedRanking.append((origin, "origin"))
    # step 2: add the on-axis locations
    for axisName in axisOrder:
        for loc in sortedOnAxis.get(axisName, []):
            ranking.append(loc)
            annotatedRanking.append((loc, "sortedOnAxis"))

    # step 3: add the aligned offaxis locations
    for loc in aligned_offAxis:
        # do these need more specific ordering?
        # for support in findOpenNodes(ranking+[loc], axisOrder):
        #     ranking.append(support)
        #     annotatedRanking.append((support, "freeze aligned_onAxis"))
        ranking.append(loc)
        annotatedRanking.append((loc, "aligned_offAxis"))

    # somewhere here 
    # 1. we need to be able to flag which supports are
    #     invalidated by a new master.
    # 2. then we need to be able to calculate new values
    #     for these supports.
    # 3. and for that we'd need a different m
    
    # step 3.1 add the partially aligned onaxis locations
    for loc in partiallyAligned_onAxis:
        # for support in findOpenNodes(ranking+[loc], axisOrder):
        #     ranking.append(support)
        #     annotatedRanking.append((support, "freeze partiallyAligned_onAxis"))
        #     addedSupports.append(support)
        ranking.append(loc)
        annotatedRanking.append((loc, "partially aligned_onAxis"))

    # step 4: add the partially aligned offaxis locations
    # check for open nodes
    for loc in partiallyAligned_offAxis:
        for support in findOpenNodes(ranking+[loc], axisOrder):
            ranking.append(support)
            addedSupports.append(support)
            annotatedRanking.append((support, "freeze partiallyAligned_offAxis"))
        ranking.append(loc)
        annotatedRanking.append((loc, "partiallyAligned_offAxis"))

    # step 5: add the non-aligned offaxis locations, closest to origin first
    proximity = {}
    for loc in nonAligned_offAxis:
        d = ranking[0].distance(loc)
        if not d in proximity:
            proximity[d] = []
        proximity[d].append(loc)
        proximity[d].sort()
    d = proximity.keys()
    d.sort()
    #d.reverse()
    closest = []
    for dist in d:
        for loc in proximity[dist]:
            closest.append(loc)
    for loc in closest:
        for support in findOpenNodes(ranking+[loc], axisOrder):
            ranking.append(support)
            addedSupports.append(support)
            annotatedRanking.append((support, "freeze nonAligned_offAxis"))
        ranking.append(loc)
        annotatedRanking.append((loc, "nonAligned_offAxis"))
    if verbose:
        # print something for the humans
        print("origin", origin)
        print("allOnAxis", allOnAxis)
        print("allOffAxis", allOffAxis)
        print("aligned_offAxis", aligned_offAxis)
        print("partiallyAligned_onAxis", partiallyAligned_onAxis)
        print("partiallyAligned_offAxis", partiallyAligned_offAxis)
        print("nonAligned_offAxis", nonAligned_offAxis)
        print("addedSupports", len(addedSupports))
    
    return annotatedRanking
