import time
from pprint import pprint

__all__ = ["StorageObject", "StorageContainer"]

class StorageObject(object):
    def __init__(self, name, thing=None):
        self.createdTime = time.time()
        self.identifier = name
        self.thing = thing
    
    def __repr__(self):
        return "<StorageObject %s %3.6f>" % (str(self.identifier), self.createdTime)

class StorageContainer(dict):
    def __init__(self):
        self.objectChanges = {}
        #self.objects = {}
        
    def notifyAboutChange(self, identifier=None):
        # keep track of when glyphs were changed last
        if identifier is None:
            self.reset()
            return
        if identifier not in self:
            return None
        t = time.time()
        self.objectChanges[identifier] = t
        return t
    
    def checkForChange(self, identifier):
        if identifier in self.objectChanges:
            return self.objectChanges[identifier]
        return self.notifyAboutChange(identifier)
    
    def getStoredObject(self, identifier):
        # stored object will have the tag?
        if identifier in self:
            obj = self[identifier]
            cachedTime = self.checkForChange(identifier)
            if obj.createdTime == cachedTime:
                # this is a good one
                return True, obj.thing
            elif obj.createdTime < cachedTime:
                # nope, it's gone stale, get rid of it right now
                self.removeObject(identifier)
                return False, None
        # inknown thing
        return False, None
    
    def storeObject(self, identifier, thing):
        obj = StorageObject(identifier, thing)
        self[identifier] = obj
        self.objectChanges[identifier] = obj.createdTime

    def removeObject(self, identifier):
        # remove one specific identifier
        if identifier in self:
            del self[identifier]
        if identifier in self.objectChanges:
            del self.objectChanges[identifier]
    
    def reset(self):
        # clear everything
        self.clear()
        self.objectChanges = {}
    
    def identifiers(self):
        return self.objectChanges.keys()
        

if __name__ == "__main__":
    class Thing(object):
        pass
    c = StorageContainer()
    
    # new object, properly stored, no changes reported
    myThing1 = Thing()
    c.storeObject("A", myThing1)
    result, obj = c.getStoredObject("A")
    assert result==True and obj == myThing1
    
    # new object, properly stored, changes reported
    myThing2 = Thing()
    c.storeObject("A", myThing2)
    c.notifyAboutChange("A")
    c.notifyAboutChange("B")
    c.notifyAboutChange("A")
    result, obj = c.getStoredObject("A")
    assert result==False and obj == None
    
    # request for unknown object
    result, obj = c.getStoredObject("D")
    assert result==False and obj == None
    
    # record change to unknown object
    assert c.notifyAboutChange("F") == None
    
    # make sure the other inquiries did not pollute the data
    c.storeObject("A", myThing2)
    result, obj = c.getStoredObject("A")
    assert len(c.objectChanges.keys()) == 1
    assert list(c.objectChanges.keys())[0] == "A"
    c.removeObject("A")
    result, obj = c.getStoredObject("A")
    assert result==False and obj == None
    
    c.storeObject("A", myThing2)
    print(c.items())
    c.clear()
    print(c.items())